//! \file    CtinWindow.cpp
//! \brief   Implementation of the Ctin window.
//! \version 2009/11/09 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane & MAYOMBO Alexis.
/*
  Nom du Fichier: SpaceWindow.cpp
  Nom de classe: SpaceWindow
  Role:
  Nombre de methodes:
  Nombre d'attributs:
  */

#include "Include/SpaceWindow.hpp"
#include "Include/TrajWindow.hpp"
/*!
 * \class SpaceWindow
 * \brief The cpp file of the SpaceWindow class for visual_YAO.
 */

using namespace std;

/******************************************* Contructeur ***************************************************/
SpaceWindow::SpaceWindow(QStringList *trajData, QWidget *parent) : QWidget(parent) {       /*!< The constructor. */

    this->setWindowTitle("Directive Space.");                     /*!< To set the window's title. */
    isCalled = false;
	//QStringList localTrajData = trajData;
 	compteur                                = 0;	
	m_spaceNameLabel			= new QLabel("Nom de l'espace: ");           /*!< Label of the QLineEdit. */
    	m_spaceTypeLabel		= new QLabel("Type de l'espace: ");           /*!< The QComboBox's type label. */
    	m_spaceD1Label          = new QLabel("Dimension 1: ");    /*!< Label of the 1st dimension. */
    	m_spaceD2Label			= new QLabel("Dimension 2: ");
	m_spaceD3Label				= new QLabel("Dimension 3: ");
    	m_spaceTrajLabel		= new QLabel("Trajectoire: ");
    
    	m_spaceNameLine     	= new QLineEdit(this);                   /*!< Name of the QLineEdit. */
    	m_spaceType         	= new QLineEdit("M",this);                   /*!< Type of the QComboBox. */
        	m_spaceType->setEnabled(false);
   	 m_spaceD1              	= new QComboBox(this);                   /*!< Name of the 1st dimension. */
        	m_spaceD1->setEditable(true);
    	m_spaceD2           	= new QComboBox(this);                   /*!< Name of the 2nd dimension. */
        m_spaceD2->setEditable(true);
    	m_spaceD3           	= new QComboBox(this);                   /*!< Name of the 3rd dimension. */
        m_spaceD3->setEditable(true);
    	m_spaceTrajname     	= new QComboBox(this);                   /*!< Name of the selected traj. Must be declared */
        m_spaceTrajname->setEditable(true);

    	m_spaceAddButton       		= new QPushButton("Ajouter", this);      /*!< The button "add". */
    	m_spaceEnregButton     		= new QPushButton("Enregistrer",this);   /*!< The button "save". */
			m_spaceEnregButton->setDisabled(true);
   	m_spaceDeleteButton    		= new QPushButton("Supprimer", this);    /*!< The button "delete". */
			m_spaceDeleteButton->setDisabled(true);
    	m_spaceHelpButton      		= new QPushButton("De l'aide ?", this);
    	m_spaceQuitButton      		= new QPushButton("Quitter", this);      /*!< The button "Quit". */

    	m_spaceTableView         	= new QTableWidget(0, 6, this); /*!< Name of the QTableWidget's object in whitch the objects are listed for the viewer. */
        m_horizontalHeader << "Nom" << "Type" << "Dim 1" << "Dim 2" << "Dim 3" << "Nom traj";
        
		m_spaceTableView->setHorizontalHeaderLabels(m_horizontalHeader);
        	m_spaceTableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        	m_spaceTableView->setShowGrid(false);
        	m_spaceTableView->setSelectionBehavior(QAbstractItemView::SelectRows);


   	m_spaceLayout1V        = new QVBoxLayout;
        	m_spaceLayout1V->addWidget(m_spaceAddButton);
        	m_spaceLayout1V->addWidget(m_spaceDeleteButton);
        	m_spaceLayout1V->addWidget(m_spaceHelpButton);

   	m_spaceLayout1H        = new QHBoxLayout;                       /*!< The 1st horizontal Layout's name. */
        	m_spaceLayout1H->addWidget(m_spaceTableView);
        	m_spaceLayout1H->addLayout(m_spaceLayout1V);

   	m_spaceLayout2H        = new QHBoxLayout;                       /*!< The 2nd horizontal Layout's name. */
		m_spaceLayout2H->addWidget(m_spaceEnregButton);
	    	m_spaceLayout2H->addWidget(m_spaceQuitButton);
	    
	m_spaceLayoutG		  = new QGridLayout;
		m_spaceLayoutG->addWidget(m_spaceNameLabel, 0, 1);		/*!< Adding in the main layout: the space's name. */
		m_spaceLayoutG->addWidget(m_spaceNameLine, 0, 2, Qt::AlignLeft); 
		m_spaceLayoutG->addWidget(m_spaceTypeLabel, 1, 1);		/*!< Adding in the main layout: the space's type. */
		m_spaceLayoutG->addWidget(m_spaceType, 1, 2, Qt::AlignLeft);
		m_spaceLayoutG->addWidget(m_spaceD1Label, 2, 1);		/*!< Adding in the main layout: the space's 1st dim. */
		m_spaceLayoutG->addWidget(m_spaceD1, 2, 2, Qt::AlignLeft);
		m_spaceLayoutG->addWidget(m_spaceD2Label, 3, 1);
		m_spaceLayoutG->addWidget(m_spaceD2, 3, 2, Qt::AlignLeft);
		m_spaceLayoutG->addWidget(m_spaceD3Label, 4, 1);
		m_spaceLayoutG->addWidget(m_spaceD3, 4, 2, Qt::AlignLeft);
		m_spaceLayoutG->addWidget(m_spaceTrajLabel, 5, 1);
		m_spaceLayoutG->addWidget(m_spaceTrajname, 5, 2, Qt::AlignLeft);
		m_spaceLayoutG->addWidget(m_spaceHelpButton, 7, 0);
		m_spaceLayoutG->addWidget(m_spaceTableView, 6, 1, 4, 4);		
		m_spaceLayoutG->addWidget(m_spaceAddButton, 7, 5);
		m_spaceLayoutG->addWidget(m_spaceDeleteButton, 8, 5);
		m_spaceLayoutG->addLayout(m_spaceLayout2H, 10, 1, 1, 4);
		
	this->setLayout(m_spaceLayoutG);                            /*!< Sets m_layout1Space as the main layout */
   
	connect(m_spaceAddButton, SIGNAL(clicked()), this, SLOT(addSpaceRow())); /*!< Adds rows in the tableview after clicking on the button "Add"*/
   	connect(m_spaceDeleteButton, SIGNAL(clicked()), this, SLOT(deleteRow()));
   	connect(m_spaceQuitButton, SIGNAL(clicked()), this, SLOT(close())); /*!< Will be closed after a click on the button "Quit" */
   	connect(m_spaceEnregButton, SIGNAL(clicked()), this, SLOT(saveInYaoStructure()));
   	connect(m_spaceHelpButton, SIGNAL(clicked()), this, SLOT(callSpaceHelp()));

    	
   	this->setFixedWidth(baseSize().width() + 610);
}
//------------------------------------  End of the contructor declaration.---------------------------------------
/*************************************  Implementation of function addSpaceRow *************************************************/
void SpaceWindow::addSpaceRow() {

	addRow();
	testContenu(compteur);	

	//static int i = 1;
        //static int j = 1;

	string name   = m_spaceNameLine->text().toStdString(),
         d1   = m_spaceD1->currentText().toStdString(),
         d2   = m_spaceD2->currentText().toStdString(),
	 	d3   = m_spaceD3->currentText().toStdString(),
	 	traj = m_spaceTrajname->currentText().toStdString();

//for (int i = 0; name.size(); i++) {

if (!isString(name)) {
	QMessageBox::about(this, tr("Erreur de saisie!"),
                               tr("<p>Le premier champ contient au moins un caractere non conforme!<p> Les Cliquer sur <b>ok</b> pour fermer cette fenetre."));
             return;
        }
if (!isInt(d1)) {
	QMessageBox::about(this, tr("Erreur de saisie!"),
                               tr("<p>Seuls les nombres entiers sont acceptes au niveau du champ D1! <p> Cliquer sur <b>ok</b> pour fermer cette fenetre."));
             return;
        }
if (!isInt(d2)) {
	QMessageBox::about(this, tr("Erreur de saisie!"),
                               tr("<p>Seuls les nombres entiers sont acceptes au niveau du champ D2! <p> Cliquer sur <b>ok</b> pour fermer cette fenetre."));
             return;
        }
if (!isInt(d3)) {
	QMessageBox::about(this, tr("Erreur de saisie!"),
                               tr("<p>Seuls les nombres entiers sont acceptes au niveau du champ D3! <p> Cliquer sur <b>ok</b> pour fermer cette fenetre."));
             return;
        }


if (!isString(traj)) {
	QMessageBox::about(this, tr("Erreur de saisie!"),
                               tr("<p>Vous avez saisi au moins un caractere non conforme au niveau du champ de la trajectoire!<p> Cliquer sur <b>ok</b> pour fermer cette fenetre."));
             return;
        }
 
    static int row = 0;
            int l = 0;

if (l >= 0)  {
            if (isCalled) {--row ; isCalled = false; }
            m_spaceTableView->setItem(row,0, new QTableWidgetItem(m_spaceNameLine->text()));
            m_spaceTableView->setItem(row,1, new QTableWidgetItem(m_spaceType->text()));
            m_spaceTableView->setItem(row,2, new QTableWidgetItem(m_spaceD1->currentText()));
            m_spaceTableView->setItem(row,3, new QTableWidgetItem(m_spaceD2->currentText()));
            m_spaceTableView->setItem(row,4, new QTableWidgetItem(m_spaceD3->currentText()));
            m_spaceTableView->setItem(row,5, new QTableWidgetItem(m_spaceTrajname->currentText()));

            m_localData.push_back(m_spaceNameLine->text() + " " + m_spaceType->text() + " " + m_spaceD1->currentText() +
                                                          " " + m_spaceD2->currentText() + " " + m_spaceD3->currentText() +
                                                          " " + m_spaceTrajname->currentText()); //<< m_spaceNameLine->text() << m_spaceType->text();
           row++;
        }
	//m_spaceDeleteButton->setEnabled(true);
	toInitialize();
	//testPresenceLigne();
        


}
/************************************* Implementation of function addRow **************************************************/
void SpaceWindow::addRow() {

	int row = m_spaceTableView->rowCount();
	m_spaceTableView->insertRow(row);
	compteur ++;

}

/************************************** Implementation of function deleteRow **********************************************/
void SpaceWindow::deleteRow() {

    	m_spaceTableView->removeRow(m_spaceTableView->currentRow());
	//testPresenceLigne();
    	
    isCalled = true;
    -- compteur;
    testContenu(compteur);
}

/************************************** Implementation of function saveInYaoStructure() ***********************************/
void SpaceWindow::saveInYaoStructure() {

	//for (QStringList::iterator it = m_localData.begin(); it != m_localData.end(); it++)
	//	m_spaceData.push_back(*it);
	m_spaceData = m_localData;
}

/****************************************************************************************************************************/
QStringList SpaceWindow::getData() {

    //QStringList test;
     return m_spaceData;
}

/****************************************************************************************************************************/
void SpaceWindow::toInitialize() {
	
	m_spaceNameLine->clear();
	m_spaceD1->clearEditText();
	m_spaceD2->clearEditText();
	m_spaceD3->clearEditText();
	m_spaceTrajname->clearEditText();

}
/****************************************************************************************************************************/
bool SpaceWindow::isString(string aString) {
//if (aString.length() > 1) {
 for (int cpt = 0; aString.size(); cpt++) { 
   if((aString[cpt] >= 'a' && aString[cpt] <= 'z') || (aString[cpt] >= 'A' && aString[cpt] <= 'Z') || (aString[cpt] >= '0' && aString[cpt] <= '9') ||
      (aString[cpt] == '/') || (aString[cpt] == '_') || (aString[cpt] == '.') || (aString[cpt] == ' '))
	return true;
   else
   	return false;        
	}
//}   
//else
//	{
//	QMessageBox::about(this, tr("Erreur de saisie!"),
  //                             tr("<p>Chaine trop courte!<p> Cliquer sur <b>ok</b> pour fermer cette fenetre."));
    //         //return;
      //  }

}


/****************************************************************************************************************************/
bool SpaceWindow::isInt(string aStringToInt) {

 for (int cpt = 0; aStringToInt.size(); cpt++) { 
   if(aStringToInt[cpt] >= '0' && aStringToInt[cpt] <= '9')
	return true;
   else
   	return false;        
	}
}
/********************************************* Implémentation du slot callSpaceHelp ***************************************/
void SpaceWindow::callSpaceHelp() {

    QMessageBox::about(this, tr("Aide sur la directive space."),("<p> La directive defval permet (a l'instar de la directive #define de c/c++) d'associer un nom symbolique a une valeur."
                                                                  "<p><u>Exemple 1</u>: deval szx 12 <p><u>Exemple 2</u>: defval szy \"chaine\"<p> <u>Exemple 3</u>: defval szz szx"));

}

/****************************************************************************************************************************/
void SpaceWindow::testPresenceLigne() {

    if (m_localData.isEmpty()) 
      {
       	 	m_spaceDeleteButton->setDisabled(true);
	 	m_spaceEnregButton->setDisabled(true);
	}
    else
        { 
		m_spaceDeleteButton->setEnabled(true);
		m_spaceDeleteButton->setDisabled(true);
	}
}
/**************************************************************************/
 bool SpaceWindow::testContenu(int anInt) {

	if (anInt != 0) {
		m_spaceEnregButton->setEnabled(true);
		m_spaceDeleteButton->setEnabled(true);
	}
	else
	{
		m_spaceEnregButton->setDisabled(true);
		m_spaceDeleteButton->setDisabled(true);
	}
}

